@@ -39,7 +39,7 @@ __author__ = "Michael Imelfort"
 __copyright__ = "Copyright 2012"
 __credits__ = ["Michael Imelfort"]
 __license__ = "GPL3"
-__version__ = "0.1.0"
+__version__ = "0.1.1"
 __maintainer__ = "Michael Imelfort"
 __email__ = "mike@mikeimelfort.com"
 __status__ = "Development"
@@ -98,13 +98,24 @@ class Mc2kOptionsParser():
 
     def Mc2kAlign(self, options):
         """Align Command"""
-        HA = resultsParser.HMMAligner(prefix=options.prefix)
+        if hasattr(options, 'separate'):
+            HA = resultsParser.HMMAligner(options.prefix,
+                      options.separate,
+                      options.consensus,
+                      options.out_format
+                      )
+        else:
+            HA = resultsParser.HMMAligner(options.prefix)
+        bh = False
+        if hasattr(options, 'best_alignment'):
+            bh=True
         HA.makeAlignments(options.out_folder,
                           options.hmm,
                           eCO=options.e_value,
                           lengthCO=options.length,                              
                           verbose=options.verbose,
-                          prefix=options.prefix
+                          prefix=options.prefix,
+                          bestHit=bh
                           )
 
     def parseOptions(self, options):
